/* bb-ccnet.c
 * last modified : 07/12/06 18:25:08 
 *
 * TOYO CC/NET CAMAC Access Module
 * Origiranl version = pcc.c by Yasu-san
 *
 * Hidetada Baba (RIKEN)
 * baba@ribf.riken.jp
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/errno.h>
#include <asm/io.h>

#include "pcc.h"

#if LINUX_VERSION_CODE >=  0x020410
  MODULE_LICENSE("GPL");
#endif

int detect_pci_device(unsigned vendor,unsigned device);

struct pcc_device{
  char *io_base;
  unsigned int irq;
  char *name;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
  wait_queue_head_t waitqTx;
  wait_queue_head_t waitqRx;
  wait_queue_head_t waitqInt;
#else
  struct wait_queue *waitqTx;
  struct wait_queue *waitqRx;
  struct wait_queue *waitqInt;
#endif
  struct pccreg pccreg;
  int statusRxInt, statusTxInt, statusIntInt;
  int int_write_counter;
  int int_read_counter;
  int read_flag;
  int dma_flag;
};

static int girq;
static struct pcc_device pccdev;

#include "ccnet.c"

int init_module(void){
    int ret;

    ret = detect_pci_device(PCI_VENDOR_ID_PCC, PCI_DEVICE_ID_PCC);
    if(ret){
      printk("CCNET Can't find.\n");
      return ret;
    }
    printk("CCNET found at ioport 0x%p on irq %d.\n",pccdev.io_base, pccdev.irq);

    if(ccnet_init_register()){
      printk("CCNET: Error in init_register\n");
      return -ENODEV;
    }

    girq = pccdev.irq;

    return 0;
}

void cleanup_module(void){
}

int detect_pci_device(unsigned vendor,unsigned device){
    struct pci_dev *dev;

    if (!pcibios_present()){
      printk("CCNET: unable to find PCI bios.\n");
      return -ENODEV;
    }

    dev = NULL;
    dev = pci_find_device(vendor, device, dev);

    if(dev == NULL){
      return -ENODEV;
    }

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
    pci_enable_device(dev);
    pccdev.io_base = (char *)pci_resource_start(dev, 0);
#else
    pccdev.io_base = (char *)(dev->base_address[0] & PCI_BASE_ADDRESS_MEM_MASK);
#endif

    pccdev.irq = dev->irq;

    pccdev.pccreg.TxData1 = (int)(pccdev.io_base + TXDATA1);
    pccdev.pccreg.TxData2 = (int)(pccdev.io_base + TXDATA2);
    pccdev.pccreg.TxControl = (int)(pccdev.io_base + TXCONTROL);
    pccdev.pccreg.TxStatus = (int)(pccdev.io_base + TXSTATUS);
    pccdev.pccreg.TxAddress = (int)(pccdev.io_base + TXADDRESS);
    pccdev.pccreg.TxPresetCount = (int)(pccdev.io_base + TXPRESETCOUNT);
    pccdev.pccreg.TxActualCount = (int)(pccdev.io_base + TXACTUALCOUNT);
    pccdev.pccreg.TxFifoCount = (int)(pccdev.io_base + TXFIFOCOUNT);
    pccdev.pccreg.RxData1 = (int)(pccdev.io_base + RXDATA1);
    pccdev.pccreg.RxData2 = (int)(pccdev.io_base + RXDATA2);
    pccdev.pccreg.RxControl = (int)(pccdev.io_base + RXCONTROL);
    pccdev.pccreg.RxStatus = (int)(pccdev.io_base + RXSTATUS);
    pccdev.pccreg.RxAddress = (int)(pccdev.io_base + RXADDRESS);
    pccdev.pccreg.RxPresetCount = (int)(pccdev.io_base + RXPRESETCOUNT);
    pccdev.pccreg.RxActualCount = (int)(pccdev.io_base + RXACTUALCOUNT);
    pccdev.pccreg.RxFifoCount = (int)(pccdev.io_base + RXFIFOCOUNT);
    pccdev.pccreg.System = (int)(pccdev.io_base + SYSTEM);
    pccdev.pccreg.IntData1 = (int)(pccdev.io_base + INTDATA1);
    pccdev.pccreg.IntData2 = (int)(pccdev.io_base + INTDATA2);
    pccdev.pccreg.IntControl = (int)(pccdev.io_base + INTCONTROL);
    pccdev.pccreg.IntStatus = (int)(pccdev.io_base + INTSTATUS);
    pccdev.pccreg.IntFifoCount = (int)(pccdev.io_base + INTFIFOCOUNT);

    return 0;
}

int ccnet_get_irq(void){
  return girq;
}
