/* ccnet.c
 * last modified : 07/12/14 13:09:01 
 *
 * TOYO CC/NET driver
 * Original = pcc.c by Yasu-san
 *
 * Hidetada Baba (RIKEN)
 * baba@ribf.riken.jp
 *
 */

int ccnet_init_register(void);
int ccnet_check_done(int try);
int ccnet_check_emp_fifo(void);
int ccnet_check_full_fifo(void);
int ccnet_check_lam(void);
void ccnet_read_lam(int *data);
int ccnet_control(int cnaf);
int ccnet_read16(int cnaf,short *data);
int ccnet_read24(int cnaf,short *data);
int ccnet_write16(int cnaf,short *data);
int ccnet_write24(int cnaf,int *data);
int ccnet_block_read16(int mode,int cnaf,short *data,int count);
int ccnet_block_read24(int mode,int cnaf,unsigned int *data,int count);
int ccnet_dma_block_read16(int mode,int cnaf,int count);
int ccnet_dma_block_read24(int mode,int cnaf,int count);
void ccnet_crate_reset(void);
void ccnet_rfs_enable_interrupt(void);
void ccnet_rfs_disable_interrupt(void);
void ccnet_pci_enable_interrupt(void);
void ccnet_pci_clear_interrupt(void);
void ccnet_crate_enable_lam(int lammask);
void ccnet_crate_disable_lam(void);
void ccnet_crate_z(int c);
void ccnet_crate_c(int c);
void ccnet_crate_seti(int c);
void ccnet_crate_deli(int c);
void ccnet_get_csrdata(int *ret);
void ccnet_get_bmcsdata(int *ret);

#define CNAFGEN(c,n,a,f) ((((n<<8) | a) << 8) | f)
#define NAFGEN(n,a,f)    ((((n<<8) | a) << 8) | f)

static int lmsk;

static int needed_pages;
static char *read_data_buf;
static char *write_data_buf;
static int *cmdbuf;
static int *rplybuf;
static int timeout;

static inline int __get_order(unsigned long size){
  int order;

  size = (size-1) >> (PAGE_SHIFT-1);
  order = -1;
  do {
    size >>= 1;
    order++;
  } while (size);
  return order;
}

int ccnet_init_register(void){

  outl(SYS_RESET|SYS_CAMAC_FRAME_SIZE, pccdev.pccreg.System);
  outl(SYS_READY|SYS_CAMAC_FRAME_SIZE, pccdev.pccreg.System);

  needed_pages = __get_order(MAX_BUFFER_SIZE);
  write_data_buf = (char *)__get_free_pages(GFP_KERNEL, needed_pages);
  if( !write_data_buf ) {
    printk("init_module:cound not get free pages for write...\n");
    return(-ENOMEM);
  }
  read_data_buf = (char *)__get_free_pages(GFP_KERNEL, needed_pages);
  if( !read_data_buf ) {
    printk("init_module:cound not get free pages for read...\n");
    return(-ENOMEM);
  }

  cmdbuf = (int *)__get_free_pages(GFP_KERNEL, needed_pages);
  if( !cmdbuf ) {
    printk("init_module:cound not get free pages for cmdbuf...\n");
    return(-ENOMEM);
  }
  rplybuf = (int *)__get_free_pages(GFP_KERNEL, needed_pages);
  if( !rplybuf ) {
    printk("init_module:cound not get free pages for rplybuf...\n");
    return(-ENOMEM);
  }

  outl(TC_INT_CLR, pccdev.pccreg.TxControl);
  outl(RC_INT_CLR, pccdev.pccreg.RxControl);
  outl(IC_INT_CLR, pccdev.pccreg.IntControl);

  ccnet_crate_deli(0);  // Delete Inhibit

  return 0;
}

short ccnet_dec(int val, short *q, short *x){
  short sval;

  sval = val & 0xffffff;
  *q = 0x1 & (val >> 24);
  *x = 0x1 & (val >> 25);

  return sval;
}

int ccnet_put_data(int cnaf, int val){
  int cmd, status;

  cmd = 0xE0000000 | cnaf;
  val = val & 0xffffff;

  timeout = 0;
  do {
    status = inl(pccdev.pccreg.TxFifoCount);
    timeout++;
  } while ( (status > MAX_FIFO_FRAME-2) && (timeout != PCC_TIMEOUT_PIO ) );
  if( timeout == PCC_TIMEOUT_PIO ) {
    printk("CCNET : put_data:timeout_pio:error...\n");
    return -ETIME;
  }
  outl(val, pccdev.pccreg.TxData1);
  outl(cmd, pccdev.pccreg.TxData2);

  return 0;
}

int ccnet_get_data(void){
  int status, rply, val;

  timeout = 0;
  do {
    status = inl(pccdev.pccreg.RxFifoCount);
    timeout++;
  } while ( (status < 2) && (timeout < PCC_TIMEOUT_PIO ));
  if( timeout == PCC_TIMEOUT_PIO ) {
    printk("CCNET : get_dat:timeout_pio:error...\n");
    return -ENODATA;
  }
  val = inl(pccdev.pccreg.RxData1);
  rply = inl(pccdev.pccreg.RxData2);

  outl(RC_INT_CLR, pccdev.pccreg.RxControl);

  return val;
}


int ccnet_control(int cnaf){
  int ret;

  ccnet_put_data(cnaf, 0);
  ret = ccnet_get_data();

  return ret;
}

int ccnet_read16(int cnaf,short *data){
  int ret;

  ccnet_put_data(cnaf, 0);
  ret = ccnet_get_data();

  *data = ret & 0x0000ffff;

  return ret;
}
int ccnet_read24(int cnaf,short *data){
  int ret;

  ccnet_put_data(cnaf, 0);
  ret = ccnet_get_data();

  *data = ret & 0x00ffffff;

  return ret;
}
int ccnet_write16(int cnaf,short *data){
  int ret;

  ccnet_put_data(cnaf, *data);
  ret = ccnet_get_data();

  return ret;
}
int ccnet_write24(int cnaf,int *data){
  int ret;

  ccnet_put_data(cnaf, *data);
  ret = ccnet_get_data();

  return ret;
}


void ccnet_crate_z(int c){
  ccnet_put_data(NAFGEN(25, 0, 17), 0);
  ccnet_get_data();
}

void ccnet_crate_c(int c){
  ccnet_put_data(NAFGEN(25, 0, 16), 0);
  ccnet_get_data();
}

void ccnet_crate_seti(int c){
  ccnet_put_data(NAFGEN(25, 0, 26), 0);
  ccnet_get_data();
}

void ccnet_crate_deli(int c){
  ccnet_put_data(NAFGEN(25, 0, 24), 0);
  ccnet_get_data();
}

int ccnet_check_done(int try){
  return 0;
}
int ccnet_check_emp_fifo(void){
  return 0;
}

int ccnet_check_full_fifo(void){
  return 0;
}

int ccnet_check_lam(void){
  return inl(pccdev.pccreg.IntStatus) & IS_INT;
}

void ccnet_read_lam(int *data){
}


int ccnet_block_read16(int mode,int cnaf,short *data,int count){
  return 0;
}

int ccnet_block_read24(int mode,int cnaf,unsigned int *data,int count){
  return 0;
}

int ccnet_dma_block_read16(int mode,int cnaf,int count){
  return 0;
}

int ccnet_dma_block_read24(int mode,int cnaf,int count){
  return 0;
}

void ccnet_crate_reset(void){
}

void ccnet_rfs_enable_interrupt(void){
  ccnet_put_data(NAFGEN(25, 1, 26), 0);
  ccnet_get_data();

  pccdev.int_read_counter = 0;
  outl(IC_INT_ENABLE | IC_CLR_FIFO, pccdev.pccreg.IntControl);
}

void ccnet_rfs_disable_interrupt(void){
  ccnet_put_data(NAFGEN(25, 1, 24), 0);
  ccnet_get_data();

  outl(IC_INT_CLR | IC_CLR_FIFO, pccdev.pccreg.IntControl);

  outl(0, pccdev.pccreg.IntControl);

  pccdev.int_read_counter = 0;
}

void ccnet_pci_enable_interrupt(void){
}

void ccnet_pci_clear_interrupt(void){
}

void ccnet_crate_enable_lam(int lammask){
  lmsk = lammask & 0x00ffffff;
  ccnet_put_data(NAFGEN(25, 1, 16), lmsk);
  ccnet_get_data();
}

void ccnet_crate_disable_lam(void){
}

void ccnet_get_csrdata(int *ret){
}

void ccnet_get_bmcsdata(int *ret){
}

void ccnet_crate_define_lam(int n){
  lmsk = (1 << (n - 1)) & 0x00ffffff;
  ccnet_put_data(NAFGEN(25, 1, 16), lmsk);
  ccnet_get_data();
}
