/* bb-k2915.c
   K3922 + K2915 CAMAC Access Module
*/

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/errno.h>
#include <asm/io.h>

#include "k2915.c"
#define cc_basea 0
#define cc_baseb 1

#if LINUX_VERSION_CODE >=  0x020410
  MODULE_LICENSE("GPL");
#endif

int detect_pci_device(unsigned vendor,unsigned device,unsigned long *tioa,
		      unsigned long *tiob, unsigned long *tir);

static int girq;

int init_module(void){
    int ret;
    unsigned long tioa,tiob,tir;

    ret = detect_pci_device(CC_VENDOR_ID,CC_DEVICE_ID,&tioa,&tiob,&tir);
    if(ret){
      printk("K2915 Can't find.\n");
      return ret;
    }
    printk("K2915 found at ioport 0x%04lx 0x%04lx on irq %lu.\n",tioa,tiob,tir);

    k2915_init_register(tir,tioa,tiob);

    girq = tir;

    return 0;
}

void cleanup_module(void){
}

int detect_pci_device(unsigned vendor,unsigned device,unsigned long *tioa,
		      unsigned long *tiob, unsigned long *tir){
    struct pci_dev *dev;

    if (!pcibios_present()){
      printk("CC: unable to find PCI bios.\n");
      return -ENODEV;
    }

    dev = NULL;
    dev = pci_find_device(vendor, device, dev);

    if(dev == NULL){
      return -ENODEV;
    }

#if LINUX_VERSION_CODE >=  0x020400
    *tioa = dev->resource[cc_basea].start & PCI_BASE_ADDRESS_IO_MASK;
    *tiob = dev->resource[cc_baseb].start & PCI_BASE_ADDRESS_IO_MASK;
    *tir  = dev->irq;
#else
    *tioa = dev->base_address[cc_basea] & PCI_BASE_ADDRESS_IO_MASK;
    *tiob = dev->base_address[cc_baseb] & PCI_BASE_ADDRESS_IO_MASK;
    *tir  = dev->irq;
#endif

    return 0;
}

int k2915_get_irq(void){
  return girq;
}
