/*
 * SBS620 PCI-VME
 * VME Access Module
 * Hidetada Baba
 * Aug 8
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/errno.h>
#include <asm/io.h>

#if LINUX_VERSION_CODE <  0x020400
#include <linux/malloc.h>
#endif

#include "sbs620.c"

#if LINUX_VERSION_CODE >=  0x020410
MODULE_LICENSE("GPL");
#endif

int detect_pci_device();

int init_module(void){
    int ret,i;
    devicen = 0;

    ret = detect_pci_device(SBS620_VENDOR_ID,SBS620_DEVICE_ID,&sbsconfig,&devicen);
    if(ret){
      printk("SBS620 Can't find.\n");
      return ret;
    }
    for(i=0;i<devicen;i++){
      printk("%d : SBS620 found at 0x%08x 0x%08x 0x%08x 0x%08x on irq %d.\n",
	     i,sbsconfig[i].addr[0],sbsconfig[i].addr[1],
	     sbsconfig[i].addr[2],sbsconfig[i].addr[3],
	     sbsconfig[i].irq);
      if(sbs620_init_register(i) == -1){
	return -ENODEV;
      }
    }

    return 0;
}

void cleanup_module(void){
  int i;
  for(i=0;i<devicen;i++){
    kfree(vreg[i].dma_buff);
    vfree(vreg[i].csr);
    vfree(vreg[i].vme);
    vfree(vreg[i].remote);
  }
}

int detect_pci_device(unsigned vendor,unsigned device,struct pci_config *sbsconfig,int *devicen){
    struct pci_dev *dev;
    int i,j;

    if (!pcibios_present()){
      printk("SBS620: unable to find PCI bios.\n");
      return -ENODEV;
    }

    dev = NULL;
    for(j=0;j<16;j++){
      dev = pci_find_device(vendor, device, dev);
      if(dev == NULL){
	break;
      }
#if LINUX_VERSION_CODE >=  0x020400
      for(i=0;i<4;i++){
	sbsconfig[j].addr[i] = dev->resource[i].start & PCI_BASE_ADDRESS_IO_MASK;
    }
      sbsconfig[j].irq = dev->irq;
#else
      for(i=0;i<4;i++){
	sbsconfig[j].addr[i] = dev->base_address[i] & PCI_BASE_ADDRESS_IO_MASK;
      }
      sbsconfig[j].irq = dev->irq;
#endif

    }

    if(j==0){
      return -ENODEV;
    }

    *devicen = j;

    return 0;
}

int sbs620_get_irq(void){
  return sbsconfig[0].irq;
}
