/* v2718.c
 * CAEN V2718 PCI-VME
 *
 * Hidetada Baba (RIKEN)
 * baba@rarfaxp.riken.jp
 * May 17, 2006
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/errno.h>
#include <asm/io.h>
#include <asm/delay.h>

#if LINUX_VERSION_CODE <  0x020400
#include <linux/malloc.h>
#endif

#include "v2718.c"

#if LINUX_VERSION_CODE >=  0x020410
MODULE_LICENSE("GPL");
#endif

int detect_pci_device();

int init_module(void){
  int ret;
  devicen = 0;
  
  printk("V2718: init_module\n");
  ret = detect_pci_device(V2718_VENDOR_ID,V2718_DEVICE_ID,&v2718config,&devicen);
  if(ret){
    printk("V2718 : Can't find PLX9054.\n");
    return ret;
  }

  printk("V2718 found at 0x%08x 0x%08x 0x%08x on irq %d.\n",
 	   v2718config[0].addr[0], v2718config[0].addr[1],
 	   v2718config[0].addr[2], v2718config[0].irq);
  if(v2718_init_register(0) == -1){
    return -ENODEV;
  }
  
  return 0;
}

void cleanup_module(void){
  //v2718_check_interrupt();
  
  kfree(vreg[0].dma_buff);
  vfree(vreg[0].csr);
  vfree(vreg[0].plx);
}

int detect_pci_device(unsigned vendor,unsigned device,struct pci_config *v2718config,int *devicen){
  struct pci_dev *dev;
    int i, n;
    unsigned short ss_id;

    if (!pcibios_present()){
      printk("V2718: Unable to find PCI bios.\n");
      return -ENODEV;
    }

    dev = NULL;
    n = 0;
    dev = pci_find_device(vendor, device, dev);
    if(dev == NULL){
      printk("V2718: Can't find PLX9054\n");
      return -ENODEV;
    }
    
    pci_read_config_word(dev, PCI_SUBSYSTEM_ID, &ss_id);
    if(ss_id == V2718_PCI_SUBDEVICE_ID){
      n++;
    }
    if(!n){
      printk("V2718: Can't fine A2818\n");
      return -ENODEV;
    }

#if LINUX_VERSION_CODE >=  0x020400
    for(i=0;i<3;i++){
      v2718config[0].addr[i] = 
	dev->resource[i].start & PCI_BASE_ADDRESS_IO_MASK;
    }
    v2718config[0].irq = dev->irq;
#else
    for(i=0;i<3;i++){
      v2718config[0].addr[i] = 
	dev->base_address[i] & PCI_BASE_ADDRESS_IO_MASK;
    }
    v2718config[0].irq = dev->irq;
#endif

    if(n==0){
      return -ENODEV;
    }
    
    *devicen = n;
    
    return 0;
}

int v2718_get_irq(void){
  return v2718config[0].irq;
}

int v2718_get_irq_i(int dn){
  return v2718_get_irq();
}
