/*
 * Wiener VMEMM / PCIADA PCI-VME
 * VME Access Module
 * H.baba
 * May 13, 2002
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/ioctl.h>
#include <linux/sched.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/errno.h>
#include <asm/io.h>

#include "vmemm.c"

int detect_pci_device(unsigned vendor,unsigned device,unsigned long *tioa,
		      unsigned long *tiob, unsigned long *tir);

#if LINUX_VERSION_CODE >=  0x020410
  MODULE_LICENSE("GPL");
#endif

static int girq;

int init_module(void){
    int ret;
    unsigned long tioa,tiob,tir;

    ret = detect_pci_device(VMEMM_VENDOR_ID,VMEMM_DEVICE_ID,&tioa,&tiob,&tir);
    if(ret){
      printk("VMEMM Can't find.\n");
      return ret;
    }
    printk("VMEMM found at 0x%04lx 0x%04lx on irq %lu.\n",tioa,tiob,tir);

    if(vmemm_init_register(tir,tioa,tiob) == -1){
      return -ENODEV;
    }

    girq = tir;

    return 0;
}

void cleanup_module(void){
}

int detect_pci_device(unsigned vendor,unsigned device,unsigned long *tioa,
		      unsigned long *tiob, unsigned long *tir){
    struct pci_dev *dev;

    if (!pcibios_present()){
      printk("VMEMM: unable to find PCI bios.\n");
      return -ENODEV;
    }

    dev = NULL;
    dev = pci_find_device(vendor, device, dev);

    if(dev == NULL){
      return -ENODEV;
    }

#if LINUX_VERSION_CODE >=  0x020400
    *tioa = dev->resource[0].start & PCI_BASE_ADDRESS_IO_MASK;
    *tiob = dev->resource[2].start & PCI_BASE_ADDRESS_IO_MASK;
    *tir  = dev->irq;
#else
    *tioa = dev->base_address[0] & PCI_BASE_ADDRESS_IO_MASK;
    *tiob = dev->base_address[2] & PCI_BASE_ADDRESS_IO_MASK;
    *tir  = dev->irq;
#endif

    return 0;
}

int vmemm_get_irq(void){
  return girq;
}
